﻿using System;
using System.Threading.Tasks;
using System.Net.Http;
using System.Text;
using Newtonsoft.Json;
using System.Dynamic;

namespace XServerJson
{
    class Program
    {
        // Make a request on xServer with JSON, using JSON.NET and dynamic/anonymous types.
        // This means we don't have any types defined for our requests and responses and do it like JavaScript.
        // This version uses xserver2 with pascal-casing for .NET types
        // It also maps the $type Attribute required for xLocate to a property name "ObjectType"
        static void Main(string[] args)
        {
            ExecAsync().Wait();

            Console.ReadKey();
        }

        static async Task ExecAsync()
        {
            // input address for geocoding as anonymous type
            var result = await GeocodeAsync(new
            {
                // required by xLocate2, mapped to $type
                ObjectType = "SearchByAddressRequest",
                Address = new {
                    Country = "L",
                    PostalCode = "",
                    City = "Schengen",
                    Street = "Rue de la Moselle 5",
                }
            });

            // now work with the object like in JavaScript
            // You don't have intellisense, but can view the fields in the watch window of the debugger.
            Console.WriteLine($"Result is of type: {result.ObjectType}");
            foreach (var ra in result.Results)
            {
                Console.WriteLine($"Match Type: {ra.Type}, Match Quality: {ra.MatchQuality.TotalScore} " +
                    $"Lat/Lng: {ra.Location.ReferenceCoordinate.Y}/{ra.Location.ReferenceCoordinate.X}");
            }
        }

        static async Task<dynamic> GeocodeAsync(object address)
        {
            // activates mapping for $type and pascal-casing for dynamic types
            var jsonSerializerSettings = new XServerSerializerSettings();

            // Create the request Client
            var httpClient = new HttpClient();

            // When using using xServer-internet you need a token!
            httpClient.DefaultRequestHeaders.Authorization =
                new System.Net.Http.Headers.AuthenticationHeaderValue("Basic",
                    Convert.ToBase64String(Encoding.UTF8.GetBytes("xtok:<your xserver-internet token>")));

            // The content (body)
            var content = new StringContent(JsonConvert.SerializeObject(address, jsonSerializerSettings), Encoding.UTF8, "application/json");

            // Do the POST
            var response = await httpClient.PostAsync("https://xserver2-europe-eu-test.cloud.ptvgroup.com/services/rs/XLocate/2.9/searchLocations", content);

            // throws an exception if not status 200
            response.EnsureSuccessStatusCode();

            // Just return as dynamic object
            return JsonConvert.DeserializeObject<ExpandoObject>(await response.Content.ReadAsStringAsync(), jsonSerializerSettings);
        }
    }
}